%  
% ZOLTAN VIZVARI, NINA GYORFI, AKOS ODRY, ZOLTAN SARI, MIHALY KLINCSIK,
% MARIN GERGICS, LEVENTE KOVACS, ANITA KOVACS, JOZSEF PAL,
% ZOLTAN KARADI, PETER ODRY AND ATTILA TOTH:
% 
% Residual impedance rejection during ultra-low frequency bio-impedance
% spectral measurements
% 
% Submission uploaded: 06/09/2020 
%
% This script shows error analysis of ultra-low frecral BIS phantom
% measurements for 40-point spectrum

clear all; clc; close all;
j = sqrt(-1);

%% Read the 40-points BIS data and create the Z_body and Z matrices

% Read the data folder
folder_read = dir('Meas_data\40_points\*.csv');

% Set the value of reference resistor
Rref = 100; %Ohm

% Read the data
oldfolder = pwd;
cd Meas_data\40_points
for i = 1:numel(folder_read)
    
    Data = csvread(folder_read(i).name,1,0);
    
    if i == 1
        
        % Z_in_meas = zeros(size(Data,1),numel(folder_read));
        Z_body_meas = zeros(size(Data,1),numel(folder_read));
        % Z_out_meas = zeros(size(Data,1),numel(folder_read));
        Z_meas = zeros(size(Data,1),numel(folder_read));
        Fq = Data(:,1); omega = 2*pi.*Fq;
        
    else
    end
    
    U1 = Data(:,2).*exp(j.*Data(:,5));
    U2 = Data(:,3).*exp(j.*Data(:,6));
    U3 = Data(:,4).*exp(j.*Data(:,7));
    
    % Z_in_meas(:,i) = Rref.*(U1-U2);    % Eq. (3a)
    Z_body_meas(:,i) = Rref.*(U2-U3);    % Eq. (3b)
    % Z_out_meas(:,i) = Rref.*(U3-1);    % Eq. (3c)
    Z_meas(:,i) = Rref.*(U1-1);          % Eq. (3d)
    
end
cd(oldfolder)


%% Calculate the spectrum of BIS phantom

% The values of phantom components
R_in = 1e5;     % Ohm
C_in = 100e-6;  % F

R_inf = 1e3;    % Ohm
R_0 = 1.1e4;    % Ohm
C = 10e-6;      % F

R_out = 1e5;    % Ohm
C_out = 100e-6; % F

% The spectrums calculated by using frequency data from the measurements
Z_in_calc = 1./((1/R_in) + j.*omega.*C_in);              % Eq. (2a)
Z_body_calc = R_inf + 1./(1/(R_0-R_inf) + j.*omega.*C);  % Eq. (2b)
Z_out_calc = 1./((1/R_out) + j.*omega.*C_out);           % Eq. (2c)
Z_calc = Z_in_calc + Z_body_calc + Z_out_calc;           % Eq. (2d)


%% Calculate the relative errors of measured BIS values


% eps_in = 100*abs(Z_in_meas-Z_in_calc)./abs(Z_in_calc);       % Eq. (4a)
eps_body = 100*abs(Z_body_meas-Z_body_calc)./abs(Z_body_calc); % Eq. (4b)
% eps_out = 100*abs(Z_out_meas-Z_out_calc)./abs(Z_out_calc);   % Eq. (4c)
eps_Z = 100*abs(Z_meas-Z_calc)./abs(Z_calc);                   % Eq. (4d)

% The maximum values of relative errors corresponding to the whole data
% table

eps_body_max = max(max(eps_body));
eps_Z_max = max(max(eps_Z));

disp(['Based on Eq.(4b), the maximum of relative error is ',...
    num2str(eps_body_max),' % in case of 40 point spectrum data.'])

disp(['Based on Eq.(4d), the maximum of relative error is ',...
    num2str(eps_Z_max),' % in case of 40 point spectrum data.'])


%% Plot the measured and calculated BIS data

% Z_body data
for i = 1:size(Z_body_meas,2)
    
    figure;
    yyaxis left;
    loglog(omega,abs(Z_body_meas(:,i)),'LineWidth',1.5);
    hold on;
    loglog(omega,abs(Z_body_calc),'--','LineWidth',2)
    hold off;
    title(['Z_{body} data, ', num2str(i),...
        '. measurement, 40 point BIS data']);
    xlim([0.001 1e7]);
    ylim([min(abs(Z_body_meas(:,i)))*0.01,max(abs(Z_body_meas(:,i)))*10]);
    grid on;
    xlabel('\omega (1/s)'); ylabel('|Z| (\Omega)');
    
    yyaxis right;
    semilogx(omega,angle(Z_body_meas(:,i)),'LineWidth',1.5);
    hold on;
    semilogx(omega,angle(Z_body_calc),'--','LineWidth',2)
    hold off;
    ylim([-pi/2 0.1]);
    ylabel('\phi (rad)');
    
    legend({'measured |Z|','calculated |Z|','measured \phi',...
        'calculated \phi'},'Location','southwest','NumColumns',2)
    
end

% Z data
for i = 1:size(Z_meas,2)
    
    figure;
    yyaxis left;
    loglog(omega,abs(Z_meas(:,i)),'LineWidth',1.5);
    hold on;
    loglog(omega,abs(Z_calc),'--','LineWidth',2)
    hold off;
    title(['Z data, ', num2str(i),...
        '. measurement, 40 point BIS data']);
    xlim([0.001 1e7]);
    ylim([min(abs(Z_meas(:,i)))*0.01,max(abs(Z_meas(:,i)))*10]);
    grid on;
    xlabel('\omega (1/s)'); ylabel('|Z| (\Omega)');
    
    yyaxis right;
    semilogx(omega,angle(Z_meas(:,i)),'LineWidth',1.5);
    hold on;
    semilogx(omega,angle(Z_calc),'--','LineWidth',2)
    hold off;
    ylim([-pi/2 0.1]);
    ylabel('\phi (rad)');
    
    legend({'measured |Z|','calculated |Z|','measured \phi',...
        'calculated \phi'},'Location','southwest','NumColumns',2)
    
end


%% Calculate the relative errors of measured Cole-Cole parameters
%  extracted from BIS data

% Read the table of Cole-Cole parameters
oldfolder = pwd;
cd Meas_data

CC_table = readtable('40_points_CCdata.csv');

cd(oldfolder)

% Create the table of Cole-Cole parameters and relativ errors

Name = {folder_read.name};
T_40points = table('Size',[numel(Name) 9],...
    'VariableTypes',{'double','double','double','double','double',...
    'double','double','double','double'},'RowNames',Name);
T_40points.Properties.VariableNames = {'Corr','a','err_a',...
    'tau','err_tau','R_0','err_R0','R_inf','err_Rinf'};
T_40points.Corr = CC_table.Corr; 
T_40points.a = CC_table.a;
T_40points.tau = CC_table.tau;
T_40points.R_0 = CC_table.R_0;
T_40points.R_inf = CC_table.R_infty;

% For calculation of errors, the following theoretical values were added
a = 1;
tau = (R_0-R_inf)*C;

% Calculation of Cole-Cole parameter errors (%) based on (Eq. 5)
T_40points.err_a = 100*abs(T_40points.a-a)...      % Eq. (5a)
    ./T_40points.a;                                     
T_40points.err_tau = 100*abs(T_40points.tau-tau)...            % Eq. (5b)
    ./T_40points.tau;
T_40points.err_R0 = 100*abs(T_40points.R_0-R_0)...             % Eq. (5c)
    ./T_40points.R_0;
T_40points.err_Rinf = 100*abs(T_40points.R_inf-R_inf)...       % Eq. (5d)
    ./T_40points.R_inf;

T_40points
disp([' The extracted Cole-Cole parameters and the relative errors '...
'(Eq. 5) calculated from 40 point BIS data'])

disp([' The averages of extracted Cole-Cole parameters are the followings',...
    '(calculated from 40 point data):'])
disp({[' a = ',num2str(mean(T_40points.a))],...
    [' tau = ',num2str(mean(T_40points.tau))],...
    [' R_0  = ',num2str(mean(T_40points.R_0))],...
    [' R_inf = ',num2str(mean(T_40points.R_inf))]})

% Plot the fitted functions on the BIS data
for i = 1:numel(folder_read)
   
    % Set the mesured |Z_body| values
    ydata = abs(Z_body_meas(:,i));
    
    % Set the calculated |Z_body| values
    param = [T_40points.a(i),T_40points.tau(i),...
        T_40points.R_0(i),T_40points.R_inf(i)];
    ycalc = CC_fun(param,omega);
    
    % Plot
    figure;
    loglog(omega,ydata,'o','MarkerSize',5,'LineWidth',2);
    hold on;
    loglog(omega,ycalc,'LineWidth',2);
    hold off;
    title({['The fitted Cole-Cole model',...
        ' (R^2 = ',num2str(T_40points.Corr(i)),')'];...
        ['(',num2str(i),'. measurement, 40 point BIS data)'];...
        ['R_0  = ',num2str(T_40points.R_0(i)),...
        ' , R_{\infty} = ',num2str(T_40points.R_inf(i)),...
        ' , a = ',num2str(T_40points.a(i)),...
        ' , \tau = ',num2str(T_40points.tau(i))]});
    xlim([0.001 1e7]);
    ylim([min(ydata)*0.5,max(ydata)*2]);
    grid on;
    xlabel('\omega (1/s)'); ylabel('|Z| (\Omega)');
    
    legend({'Measured Impedance Values','Fitted Cole-Cole model'},...
        'Location','southwest');

end
